package org.fao.oekc.parsers;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * Parses the ASC translations file
 * @author celli
 *
 */
public class ASCSaxParser extends DefaultHandler {
	
	//mappings
	private Map<String, List<String>> code2uris;
	
	//flag for contents
	private boolean isCode;
	private boolean isURI;
	private String tmpCode;
	
	//to read the entire content
	private StringBuffer buffer;
	

	/**
	 * Initialize the parser
	 * @param code2uris the result map ASC_CODE -> Agrovoc URIs
	 */
	public ASCSaxParser(Map<String, List<String>> code2uris) {
		try {
			this.code2uris = code2uris;

			//flags
			this.isCode = false;
			this.isURI = false;
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	////////////////////////////////////////////////////////////////////
	// Event handlers.
	////////////////////////////////////////////////////////////////////
	/**
	 * Recognizes an XML element to index
	 * @param namespaceURI <code>String</code> namespace URI this element is associated with, or an empty <code>String</code>
	 * @param localName <code>String</code> name of element (with no namespace prefix, if one is present)
	 * @param rawName <code>String</code> XML 1.0 version of element name: [namespace prefix]:[localName]
	 * @param atts <code>Attributes</code> list for this element
	 * @throws <code>SAXException</code> when things go wrong
	 */
	public void startElement (String namespaceURI, String localName, String rawName, Attributes atts) throws SAXException {
		if(rawName.equalsIgnoreCase("Code")){
			this.buffer = new StringBuffer();
			this.tmpCode = null;
			this.isCode = true;
		}
		else
			if(rawName.equalsIgnoreCase("URI")){
				this.buffer = new StringBuffer();
				this.isURI = true;
			}
	}
	
	/**
	 * @param ch <code>char[]</code> character array with character data
	 * @param start <code>int</code> index in array where data starts.
	 * @param length <code>int</code> length of data in array.
	 * @throws <code>SAXException</code> when things go wrong
	 */
	public void characters(char[] ch, int start, int length) {
		//title, authors ...
		if(this.isURI || this.isCode){
			this.buffer.append(ch, start, length);
		}
	}

	/**
	 * Buffers management and end of record
	 * @param namespaceURI <code>String</code> URI of namespace this element is associated with
	 * @param localName <code>String</code> name of element without prefix
	 * @param rawName <code>String</code> name of element in XML 1.0 form
	 * @throws <code>SAXException</code> when things go wrong
	 */
	public void endElement (String namespaceURI, String localName, String rawName) throws SAXException {
		//code
		if(rawName.equalsIgnoreCase("Code") && this.isCode){
			this.isCode = false;
			String term = new String(this.buffer);
			if(term!=null && !term.trim().equals("")){
				//temporary code
				this.tmpCode = term;
			}
		} else if(rawName.equalsIgnoreCase("URI") && this.isURI){
			this.isURI = false;
			String term = new String(this.buffer);
			if(term!=null && !term.trim().equals("")){
				//add URI
				if(this.code2uris.get(this.tmpCode)==null)
					this.code2uris.put(this.tmpCode, new ArrayList<String>());
				this.code2uris.get(this.tmpCode).add(term);
			}
		}
	}
}
